library WebSearch;

uses
  Windows,
  Classes,
  Messages,
  SysUtils,
  ShellAPI,
  wbConvert,
  Resource in '.\RESOURCE\Resource.pas',
  Plugin   in '.\Plugin.pas';

{$R .\RESOURCE\WSEARCH.RES}

const
  plugin_name:     string = S_PLUGINNAME;
  plugin_version:  string = S_PLUGINVERSION;

type
  TEncodeType = (etSjis, etUtf8, etEuc);
  PWebSearchRec = ^TWebSearchRec;
  TWebSearchRec = record
    srCaption: array [0..255] of Char;
    srCommand: array [0..255] of Char;
    srEncodeType: TEncodeType;
  end;

var
  f_ItemList: TList;

function GetDLLPath: string;
var
  cPath: array[0..MAX_PATH - 1] of Char;
begin
  GetModuleFileName(HInstance, cPath, MAX_PATH);
  Result := ExtractFilePath(cPath);
end;

function AddItemList(var ItemList: TList; Caption, Command: string; EncodeType: TEncodeType): Integer;
var
  pItemRec: PWebSearchRec;
begin
  New(pItemRec);
  ZeroMemory(pItemRec, SizeOf(TWebSearchRec));
  StrCopy(pItemRec^.srCaption, PChar(Caption));
  StrCopy(pItemRec^.srCommand, PChar(Command));
  pItemRec^.srEncodeType := EncodeType;
  Result := ItemList.Add(pItemRec);
end;

procedure InsertItemList(var ItemList: TList; Idx: Integer; Caption,
                         Command: string; EncodeType: TEncodeType);
var
  pItemRec: PWebSearchRec;
begin
  New(pItemRec);
  ZeroMemory(pItemRec, SizeOf(TWebSearchRec));
  StrCopy(pItemRec^.srCaption, PChar(Caption));
  StrCopy(pItemRec^.srCommand, PChar(Command));
  pItemRec^.srEncodeType := EncodeType;
  ItemList.Insert(Idx, pItemRec);
end;

function LoadItemList(var List: TList): Integer;
var
  sIniName: string;
  sl, sline: TStringList;
  i, iEnc: Integer;
begin
  Result := -1;
  sIniName  := GetDLLPath + S_ININAME;
  if not FileExists(sIniName) then
  begin
    if FileExists(GetDLLPath + S_DEFNAME) then
      CopyFile(PChar(GetDLLPath + S_DEFNAME), PChar(sIniName), False)
    else
      Exit;
  end;
  sl := TStringList.Create;
  try
    try
      sl.LoadFromFile(sIniName);
    except
      Exit;
    end;
    sline := TStringList.Create;
    try
      for i := 0 to sl.Count - 1 do
      begin
        if Copy(sl[i], 1, 1) = S_INIREMARKCHAR then Continue;
        if sl[i] = S_MENUSEPARATOR then
        begin
          AddItemList(List, S_MENUSEPARATOR, '', TEncodeType(-1));
          Continue;
        end else
        begin
          sline.CommaText := sl[i];
          if sline.Count < 3 then Continue;
          try
            iEnc := StrToIntDef(sline[2], 0);
            AddItemList(List, sline[0], sline[1], TEncodeType(iEnc));
          except
            Continue;
          end;
        end;
      end;
    finally
      sline.Free;
    end;
  finally
    sl.Free;
  end;
  Result := List.Count;
end;

procedure SaveItemList(List: TList);
var
  sline, scap, scom: string;
  sl: TStringList;
  i, iEnc: Integer;
begin
  sl := TStringList.Create;
  try
    for i := 0 to List.Count - 1 do
    begin
      with PWebSearchRec(List[i])^ do
      begin
        scap := srCaption;
        scom := srCommand;
        iEnc := Integer(srEncodeType);
      end;
      sline := '"'   +
               StringReplace(scap, '"', '""', [rfReplaceAll, rfIgnoreCase]) +
               '","' +
               StringReplace(scom, '"', '""', [rfReplaceAll, rfIgnoreCase]) +
               '",'  + IntToStr(iEnc);
      sl.Add(sline);
    end;
    sl.SaveToFile(GetDLLPath + S_ININAME);
  finally
    sl.Free;
  end;
end;

procedure FreeItemList(var List: TList);
var
  i: Integer;
begin
  try
    for i := 0 to List.Count - 1 do
    begin
      try
        Dispose(PWebSearchRec(List[i]));
      except
        Continue;
      end;
    end;
  finally
    FreeAndNil(List);
  end;
end;

procedure CreateAndRunInternetShortcut(hwnd: THandle; cmd: Integer;
                                       ItemList: TList);
var
  iSelLength: Integer;
  pSelText: PChar;
  sSelectText, sFileName, sExecType: string;
  sl: TStringList;
begin
  sFileName := GetDLLPath + S_URLNAME;
  iSelLength := Editor_GetSelTextA(hwnd, 0, nil);
  if iSelLength = 0 then Exit;
  GetMem(pSelText, iSelLength + 1);
  try
    Editor_GetSelTextA(hwnd, iSelLength, pSelText);
    sSelectText := wbConvert.DeleteCRLF(pSelText);

    case PWebSearchRec(ItemList[cmd - 1])^.srEncodeType of
      etSjis: sSelectText := sSelectText;
      etUtf8: sSelectText := wbConvert.AnsiToUtf8(sSelectText);
      etEuc:  sSelectText := wbConvert.SJisToEuc(sSelectText);
    end;
    sSelectText := wbConvert.StrEncode(sSelectText);
    if sSelectText = '' then Exit;
    with PWebSearchRec(ItemList[cmd - 1])^ do
      sSelectText := StringReplace(srCommand, '%s',
                                   sSelectText, [rfReplaceAll]);
    sExecType := LowerCase(Copy(sSelectText, 1, 7));
    if (sExecType = 'http://') or
       (sExecType = 'file://') or
       (sExecType = 'ftp://')  then
    begin
      sl := TStringList.Create;
      try
        sl.Clear;
        sl.Add('[InternetShortcut]');
        sl.Add('URL=' + sSelectText);
        sl.SaveToFile(sFileName);
      finally
        sl.Free;
      end;
    end else
      sFileName := sSelectText;
    ShellExecute(hwnd, 'open', PChar(sFileName), nil, nil, SW_SHOW);
  finally
    FreeMem(pSelText);
  end;
end;

function CreateAndRunPopupMenuFromItemList(hwnd: THandle;
                                           ItemList: TList): Cardinal;
var
  hPopup: HMENU;
  i, iCount: Integer;
  ptCursor: TPoint;
begin
  Result := 0;
  hPopup := CreatePopupMenu;
  if hPopup = 0 then Exit;
  try
    iCount := ItemList.Count;
    if iCount > 255 then iCount := 255;
    for i := 0 to iCount - 1 do
    begin
      with PWebSearchRec(ItemList[i])^ do
      begin
        if StrPas(srCaption) = S_MENUSEPARATOR then
          AppendMenu(hPopup, MF_BYPOSITION or MF_SEPARATOR, 0, nil)
        else
          AppendMenu(hPopup, MF_STRING, i + 1, @srCaption);
      end;
    end;
    if iCount > 0 then
      AppendMenu(hPopup, MF_BYPOSITION or MF_SEPARATOR, 0, nil);
    AppendMenu(hPopup, MF_STRING, 999, PChar(S_MENU_PROPERTY));
    GetCursorPos(ptCursor);
    Result := Cardinal(TrackPopupMenu(hPopup,
                                      TPM_LEFTALIGN   or TPM_LEFTBUTTON  or
                                      TPM_RIGHTBUTTON or TPM_RETURNCMD,
                                      ptCursor.x, ptCursor.y,
                                      0, hwnd, nil));
  finally
    DestroyMenu(hPopup);
  end;  
end;

function DialogFunc(hDlg: THandle; uMsg: Longint;
                    wParam: WPARAM; lParam: LPARAM): Boolean; export; stdcall;

  function SelItem: Integer;
  begin
    Result := SendDlgItemMessage(hDlg, IDC_LISTBOX_1, LB_GETCURSEL, 0, 0);
  end;

  procedure SetEncodeType(EncodeType: TEncodeType);
  begin
    SendDlgItemMessage(hDlg, IDC_RADIO_SJIS,
                       BM_SETCHECK, Ord(EncodeType = etSjis), 0);
    SendDlgItemMessage(hDlg, IDC_RADIO_UTF8,
                       BM_SETCHECK, Ord(EncodeType = etUTF8), 0);
    SendDlgItemMessage(hDlg, IDC_RADIO_EUC,
                       BM_SETCHECK, Ord(EncodeType = etEUC),  0);
  end;

  procedure UpdateControls;
  var
    iIdx, iCnt: Integer;
    b: boolean;
  begin
    iIdx := SelItem;
    iCnt := f_ItemList.Count;
    b := (iIdx >= 0) and (iIdx < iCnt);
    EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_DEL),   b);
    EnableWindow(GetDlgItem(hDlg, IDC_RADIO_SJIS),   b);
    EnableWindow(GetDlgItem(hDlg, IDC_RADIO_UTF8),   b);
    EnableWindow(GetDlgItem(hDlg, IDC_RADIO_EUC),    b);
    EnableWindow(GetDlgItem(hDlg, IDC_EDIT_CAPTION), b);
    EnableWindow(GetDlgItem(hDlg, IDC_EDIT_COMMAND), b);
    EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_UP),    iIdx > 0);
    EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_DOWN),  iIdx < iCnt - 1);
    if b then
    begin
      with PWebSearchRec(f_ItemList[iIdx])^ do
      begin
        SetDlgItemText(hDlg, IDC_EDIT_CAPTION, @srCaption);
        SetDlgItemText(hDlg, IDC_EDIT_COMMAND, @srCommand);
        if StrPas(srCaption) = S_MENUSEPARATOR then
          SetEncodeType(TEncodeType(-1))
        else
          SetEncodeType(srEncodeType);
      end;
    end else
    begin
      SetDlgItemText(hDlg, IDC_EDIT_CAPTION, nil);
      SetDlgItemText(hDlg, IDC_EDIT_COMMAND, nil);
      SetEncodeType(TEncodeType(-1));
    end;
  end;

var
  i, iIdx: Integer;
begin
  Result := False;
  case uMsg of
    WM_INITDIALOG:
      begin
        f_ItemList := TList.Create;
        LoadItemList(f_ItemList);
        for i := 0 to f_ItemList.Count - 1 do
          with PWebSearchRec(f_ItemList[i])^ do
            SendDlgItemMessage(hDlg, IDC_LISTBOX_1,
                               LB_ADDSTRING, 0, Integer(@srCaption));
        SendDlgItemMessage(hDlg, IDC_LISTBOX_1, LB_SETCURSEL, 0, 0);
        UpdateControls;
        Result := True;
      end;
    WM_COMMAND:
      begin
        case LOWORD(wParam) of
          IDC_BUTTON_OK:
            begin
              Result := True;
              SaveItemList(f_ItemList);
              EndDialog(hDlg, IDOK);
              FreeItemList(f_ItemList);
            end;
          IDC_BUTTON_CANCEL:
            begin
              Result := True;
              EndDialog(hDlg, IDCANCEL);
              FreeItemList(f_ItemList);
            end;
          IDC_BUTTON_RESET:
            begin
              Result := True;
              if MessageBox(hDlg,
                            PChar(S_CONFIRMRESET),
                            PChar(S_CONFIRM),
                            MB_ICONINFORMATION + MB_YESNO) = IDYES then
              begin
                SendDlgItemMessage(hDlg, IDC_LISTBOX_1, LB_RESETCONTENT, 0, 0);
                FreeItemList(f_ItemList);
                DeleteFile(GetDLLPath + S_ININAME);
                f_ItemList := TList.Create;
                LoadItemList(f_ItemList);
                for i := 0 to f_ItemList.Count - 1 do
                  with PWebSearchRec(f_ItemList[i])^ do
                    SendDlgItemMessage(hDlg, IDC_LISTBOX_1,
                                       LB_ADDSTRING, 0, Integer(@srCaption));
                SendDlgItemMessage(hDlg, IDC_LISTBOX_1, LB_SETCURSEL, 0, 0);
                UpdateControls;
              end;
            end;
          IDC_LISTBOX_1:
            begin
              Result := True;
              case HIWORD(wParam) of
                LBN_SELCHANGE: UpdateControls;
              end;
            end;
          IDC_BUTTON_ADD:
            begin
              Result := True;
              iIdx := SelItem + 1;
              if iIdx >= f_itemList.Count + 1 then iIdx := f_itemList.Count;
              SendDlgItemMessage(hDlg, IDC_LISTBOX_1, LB_INSERTSTRING,
                                 iIdx, Integer(PChar(S_NEWITEM)));
              InsertItemList(f_ItemList, iIdx, S_NEWITEM, '', etEuc);
              SendDlgItemMessage(hDlg, IDC_LISTBOX_1, LB_SETCURSEL, iIdx, 0);
              UpdateControls;
            end;
          IDC_BUTTON_DEL:
            begin
              Result := True;
              iIdx := SelItem;
              if iIdx  >= 0  then
              begin
                SendDlgItemMessage(hDlg,
                                   IDC_LISTBOX_1, LB_DELETESTRING, iIdx, 0);
                Dispose(PWebSearchRec(f_ItemList[iIdx]));
                f_ItemList.Delete(iIdx);
                if iIdx > f_ItemList.Count - 1 then
                  iIdx := f_ItemList.Count - 1;
                SendDlgItemMessage(hDlg,
                                   IDC_LISTBOX_1, LB_SETCURSEL, iIdx, 0);
                UpdateControls;
              end;
            end;
          IDC_BUTTON_UP:
            begin
              Result := True;
              iIdx := SelItem;
              if iIdx > 0 then
              begin
                f_ItemList.Move(iIdx, iIdx - 1);
                SendDlgItemMessage(hDlg,
                                   IDC_LISTBOX_1, LB_DELETESTRING, iIdx, 0);
                with PWebSearchRec(f_ItemList[iIdx - 1])^ do
                  SendDlgItemMessage(hDlg, IDC_LISTBOX_1, LB_INSERTSTRING,
                                     iIdx - 1, Integer(@srCaption));
                SendDlgItemMessage(hDlg,
                                   IDC_LISTBOX_1, LB_SETCURSEL, iIdx - 1, 0);
                UpdateControls;
              end;
            end;
          IDC_BUTTON_DOWN:
            begin
              Result := True;
              iIdx := SelItem;
              if iIdx < f_ItemList.Count - 1 then
              begin
                f_ItemList.Move(iIdx, iIdx + 1);
                SendDlgItemMessage(hDlg, IDC_LISTBOX_1,
                                   LB_DELETESTRING, iIdx, 0);
                with PWebSearchRec(f_ItemList[iIdx + 1])^ do
                  SendDlgItemMessage(hDlg, IDC_LISTBOX_1, LB_INSERTSTRING,
                                     iIdx + 1, Integer(@srCaption));
                SendDlgItemMessage(hDlg, IDC_LISTBOX_1,
                                   LB_SETCURSEL, iIdx + 1, 0);
                UpdateControls;
              end;
            end;
          IDC_EDIT_CAPTION:
            begin
              Result := True;
              if f_ItemList.Count = 0 then Exit;
              iIdx := SelItem;
              if iIdx = -1 then Exit;
              case HIWORD(wParam) of
                EN_CHANGE:
                  begin
                    SendDlgItemMessage(hDlg, IDC_LISTBOX_1,
                                       LB_DELETESTRING, iIdx, 0);
                    with PWebSearchRec(f_ItemList[iIdx])^ do
                    begin
                      GetDlgItemText(hDlg, IDC_EDIT_CAPTION, srCaption, 256);
                      SendDlgItemMessage(hDlg, IDC_LISTBOX_1, LB_INSERTSTRING,
                                         iIdx, Integer(@srCaption));
                    end;
                    SendDlgItemMessage(hDlg, IDC_LISTBOX_1,
                                       LB_SETCURSEL, iIdx, 0);
                    Result := True;
                  end;
              end;

            end;
          IDC_EDIT_COMMAND:
            begin
              Result := True;
              if f_ItemList.Count = 0 then Exit;
              iIdx := SelItem;
              if iIdx = -1 then Exit;
              case HIWORD(wParam) of
                EN_CHANGE:
                  begin
                    with PWebSearchRec(f_ItemList[iIdx])^ do
                    begin
                      GetDlgItemText(hDlg, IDC_EDIT_COMMAND, srCommand, 256);
                    end;
                  end;
              end;
            end;
          IDC_RADIO_SJIS:
            begin
              Result := True;
              if f_ItemList.Count = 0 then Exit;
              iIdx := SelItem;
              if iIdx = -1 then Exit;
              with PWebSearchRec(f_ItemList[iIdx])^ do
                srEncodeType := etSjis;
            end;
          IDC_RADIO_UTF8:
            begin
              Result := True;
              if f_ItemList.Count = 0 then Exit;
              iIdx := SelItem;
              if iIdx = -1 then Exit;
              with PWebSearchRec(f_ItemList[iIdx])^ do
                srEncodeType := etUtf8;
            end;
          IDC_RADIO_EUC:
            begin
              Result := True;
              if f_ItemList.Count = 0 then Exit;
              iIdx := SelItem;
              if iIdx = -1 then Exit;
              with PWebSearchRec(f_ItemList[iIdx])^ do
                srEncodeType := etEUC;
            end;
          IDL_CAPTION:
            begin
              SetFocus(GetDlgItem(hDlg, IDC_EDIT_CAPTION));
              Result := True;
            end;
          IDL_COMMAND:
            begin
              SetFocus(GetDlgItem(hDlg, IDC_EDIT_COMMAND));
              Result := True;
            end;
        end;
      end;
  end;
end;


//  ** K{ **
// vOCs\A܂̓`FbNꂽԂ𒲂ׂ܂
function QueryStatus(hwnd: THandle; pbChecked: PBoolean): Boolean; stdcall;
begin
  Result := Editor_GetSelTextA(hwnd, 0, nil) > 1;
end;

//  ** K{ **
// ԂύXꂽƂɃCxgw肳ČĂяo܂
procedure OnEvents(hwnd: THandle; nEvent: Longint; lParam: LPARAM); stdcall;
begin
  if (nEvent and EVENT_SEL_CHANGED) <> 0 then
    Editor_UpdateToolbar(hwnd, lParam);
end;

//  ** K{ **
// vOC̃j[ACeeLXg̃\[XID擾܂
function GetMenuTextID: Longint; stdcall;
begin
  Result := IDS_MENUCAPTION;
end;

//  ** K{ **
// Xe[^Xo[eLXgƃc[o[̃c[`bvpeLXg \n 
// ̃\[XID擾܂B
function GetStatusMessageID: Longint; stdcall;
begin
  Result := IDS_STATUSBAR;
end;

function GetBitmapID: Longint; stdcall;
begin
  Result := IDB_16C_16;
end;

function PlugInProc(hwnd: THandle; nMsg: Longint;
                    wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
begin
  Result := 0;
  case nMsg of
    EP_QUERY_PROPERTIES:
      begin
        Result := Longint(True);
      end;
    EP_SET_PROPERTIES:
      begin
        DialogBox(hInstance, MAKEINTRESOURCE(IDD_PROPERTY), hwnd, @DialogFunc);
        Result := Longint(True);
      end;
    EP_GET_NAMEA:
      begin
         Result := Length(plugin_name);
         if lParam <> 0 then
           lstrcpyn(PChar(lParam), PChar(plugin_name), wParam);
      end;
    EP_GET_VERSIONA :
      begin
        Result := Length(plugin_version);
        if lParam <> 0 then
          lstrcpyn(PChar(lParam), PChar(plugin_version), wParam);
      end;
    EP_QUERY_UNINSTALL:
      begin
        Result := Longint(True);
      end;
    EP_SET_UNINSTALL:
      begin
        if MessageBox(hwnd, PChar(S_CONFIRMUNINSTALL), PChar(S_CONFIRM),
                      MB_ICONINFORMATION + MB_YESNO) = IDYES then
        begin
          DeleteFile(GetDLLPath + S_DEFNAME);
          DeleteFile(GetDLLPath + S_ININAME);
          DeleteFile(GetDLLPath + S_URLNAME);
          Result := Longint(True);
        end;
      end;
    EP_GET_BITMAP:
      begin
        if (wParam and BITMAP_LARGE) <> 0 then
        begin

          if ((wParam and BITMAP_256_COLOR)   <> 0) or
             ((wParam and BITMAP_24BIT_COLOR) <> 0) then
          begin
            if (wParam and BITMAP_DISABLED) <> 0 then
              Result := IDB_256_24_DISABLE
            else
              Result := IDB_256_24;
          end else
          begin
            if (wParam and BITMAP_DISABLED) <> 0 then
              Result := IDB_16C_24_DISABLE
            else
              Result := IDB_16C_24;
          end;
        end else
        begin
          if ((wParam and BITMAP_256_COLOR)   <> 0) or
             ((wParam and BITMAP_24BIT_COLOR) <> 0) then
          begin
            if (wParam and BITMAP_DISABLED) <> 0 then
              Result := IDB_256_16_DISABLE
            else
              Result := IDB_256_16;
          end else
          begin
            if (wParam and BITMAP_DISABLED) <> 0 then
              Result := IDB_16C_16_DISABLE
            else
              Result := IDB_16C_16;
          end;
        end;
      end;
    EP_GET_MASK:
      begin
        Result := RGB(255, 0, 255);
      end;
  end;
end;

//  ** K{ **
//  j[c[o[vOCIƂɌĂяo܂
procedure OnCommand(hwnd: THandle); stdcall;
var
  cmd: Cardinal;
  lstItem: TList;
begin
  lstItem := TList.Create;
  try
    LoadItemList(lstItem);
    cmd := CreateAndRunPopupMenuFromItemList(hwnd, lstItem);
    case cmd of
        0: Exit;
      999: PlugInProc(hwnd, EP_SET_PROPERTIES, 0, 0);
      else CreateAndRunInternetShortcut(hwnd, cmd, lstItem);
    end;
  finally
    FreeItemList(lstItem);
  end;
end;

exports
  OnCommand          index 1,
  QueryStatus        index 2,
  OnEvents           index 3,
  GetMenuTextID      index 4,
  GetStatusMessageID index 5,
  GetBitmapID        index 6,
  PlugInProc         index 7;

function DLLMain(Reason: Integer): Boolean; stdcall;
begin
  Result := True;
end;

begin
  DLLProc := @DLLMain;
  DLLMain(DLL_PROCESS_ATTACH);
end.

